#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
import re

# target = __import__("adb.py")
from androidtvremote import adb


class TestADB(unittest.TestCase):
    def setUp(self):
        self.adb = adb.ADB()

    def test_is_connected(self):
        self.assertTrue(self.adb.is_connected())

    def test_get_state(self):
        result = self.adb.get_state()
        matches = re.match(r"[\w\s]+", result)
        self.assertEqual(result, matches.group(0))

    def test_get_serialno(self):
        result = self.adb.get_serialno()
        matches = re.match(r"[\w\.\:]+", result)
        self.assertEqual(result, matches.group(0))

    # def test_stop_service(self):
    #     result = self.adb.stop_service('com.android.printspooler')

    def tearDown(self):
        pass


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

import logging
import re
import shlex
import subprocess
import sys
import time

logger = logging.getLogger(__name__)


def exec_cmd(cmd):
    """Execute the command and return clean output"""
    # logger.debug(cmd)
    cmd = shlex.split(cmd, posix="win" not in sys.platform)
    output = subprocess.check_output(cmd)
    return output.decode("utf-8").strip()

    # proc = subprocess.run(
    #     shlex.split(cmd, posix="win" not in sys.platform),
    #     check=True,
    #     capture_output=True,
    #     text=True,
    #     # shell=True,  # This is to be used with care!
    # )
    # return proc.stdout.strip()


class ADB:
    """
    Limited set of ADB commands.
    https://developer.android.com/studio/command-line/adb
    """

    serial = None
    adb_process = None

    def __init__(self):
        if sys.platform == "win32":
            self.adb_path = exec_cmd("where adb")
        else:
            self.adb_path = exec_cmd("which adb")

        if not self.adb_path:
            raise RuntimeError(
                "You need Android Platform Tools installed and available on your PATH. https://developer.android.com/studio/releases/platform-tools#download"
            )

        # https://stackoverflow.com/questions/44702657/starting-adb-daemon-python/44702825
        self.adb_process = self.start_server()

    def __str__(self):
        return self.serial or "Unknown Device"

    def cmd(self, cmd):
        """Return adb command string."""
        parts = [self.adb_path]
        if self.serial:
            parts.append(f"-s {self.serial}")
        parts.append(cmd)
        return " ".join(parts)

    def tcp(self):
        cmd = self.cmd("tcpip 5555")
        return exec_cmd(cmd)

    # General Commands -------------------------------------------------------

    def devices(self, descriptions=True):
        """Print a list of all devices. Use the -l option to
        include the device descriptions."""
        cmd = [f"{self.adb_path} devices"]
        if descriptions:
            cmd.append("-l")
        cmd = " ".join(cmd)
        return exec_cmd(cmd)

    # Connectivity -----------------------------------------------------------

    def check_connection(self, ip):
        """Check if we have device with the IP."""
        cmd = f"{self.adb_path} devices"
        if ip in exec_cmd(cmd):
            logger.info("Found device with IP %s", ip)
            return True
        logger.warning("Found no device at %s", ip)
        return False

    def connect(self, ip):
        """Connect to the device via ADB connect."""
        cmd = f"{self.adb_path} connect {ip}"
        logger.debug("Connect to ADB device %s ...", ip)
        if "failed" in exec_cmd(cmd):
            raise RuntimeError(f"Unable to connect to {ip}")
        self.serial = self.get_serialno()

    def is_connected(self):
        """Determine if device is connected."""
        cmd = f"{self.adb_path} connect {self.ip}"
        return "connected" in exec_cmd(cmd)

    def disconnect(self):
        """Disconnect device."""
        cmd = self.cmd("disconnect")
        return exec_cmd(cmd)

    # File Transfer Commands --------------------------------------------------

    def push(self, local, remote="/data/local/tmp/"):
        """Copy files and directories from the local device (computer) to
        a remote location on the device."""
        cmd = self.cmd(f"push {local} {remote}")
        return exec_cmd(cmd)

    def pull(self, remote, local, preserve_meta=False):
        """Copy remote files and directories to a device. Use the -a option
        to preserve the file time stamp and mode."""
        cmd = ["pull"]
        if preserve_meta:
            cmd.append("-k")
        cmd.append(f"{remote} {local}")
        cmd = " ".join(cmd)
        cmd = self.cmd(cmd)
        return exec_cmd(cmd)

    # App Installation Commands -----------------------------------------------

    def install(self, apk_file, replace=True):
        """Install app."""
        cmd = ["install"]
        if replace:
            cmd.append("-r")
        cmd.append(apk_file)
        cmd = " ".join(cmd)
        cmd = self.cmd(cmd)
        return exec_cmd(cmd)

    def uninstall(self, package, keep_data=False):
        """Remove this app package from the device. Add the -k option to
        keep the data and cache directories."""
        cmd = ["uninstall"]
        if keep_data:
            cmd.append("-k")
        cmd.append(package)
        cmd = " ".join(cmd)
        cmd = self.cmd(cmd)
        return exec_cmd(cmd)

    # Scripting Commands ------------------------------------------------------

    def get_state(self):
        """Print the adb state of a device. The state can be offline,
        device or no device."""
        cmd = self.cmd("get-state")
        return exec_cmd(cmd)

    def get_serialno(self):
        """ "Print the adb device serial number string."""
        if self.serial:
            return self.serial
        cmd = self.cmd("get-serialno")
        serial = exec_cmd(cmd)
        self.serial = serial
        return serial

    def get_devpath(self):
        """Print the adb device path."""
        cmd = self.cmd("get-devpath")
        return exec_cmd(cmd)

    def remount(self):
        """Remount the /system, /vendor, and /oem partitions in
        read-write mode."""
        raise NotImplementedError

    def reboot(self, mode=None):
        """Reboot the device [bootloader | recovery | sideload | sideload-auto-reboot]."""
        cmd = ["reboot"]
        if mode:
            cmd.append(mode)
        cmd = " ".join(cmd)
        cmd = self.cmd(cmd)
        return exec_cmd(cmd)

    def start_server(self):
        cmd = [self.adb_path, "start-server"]
        adb = subprocess.Popen(cmd, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
        # Give the ADB process time to start up. Don't like this, but it works.
        time.sleep(5)
        return adb

    def kill_server(self):
        cmd = self.cmd("kill-server")
        return exec_cmd(cmd)

    # Misc --------------------------------------------------------------------

    def set_home_activity(self, package, activity):
        """Set home activity."""
        cmd = self.cmd(f"shell cmd package set-home-activity {package}/{activity}")
        return exec_cmd(cmd)

    def start_app(self, package, activity, wait=True, stop=True):
        """Start app."""
        cmd = ["shell am start"]
        # wait for launch to complete
        if wait:
            cmd.append("-W")
        # force stop the target app before starting the activity
        if stop:
            cmd.append("-S")
        cmd.append(f"{package}/{activity}")
        cmd = " ".join(cmd)
        cmd = self.cmd(cmd)
        return exec_cmd(cmd)

    def stop_app(self, package):
        """Force stop everything associated with <PACKAGE>."""
        cmd = self.cmd(f"shell am force-stop {package}")
        return exec_cmd(cmd)

    def list_3rd_party_packages(self):
        """List 3rd party packages."""
        cmd = self.cmd("shell cmd package list packages -3")
        return exec_cmd(cmd)

    def kill_all(self):
        """Kill all background processes."""
        cmd = self.cmd("shell am kill-all")
        return exec_cmd(cmd)

    def stop_service(self, service):
        """Stop a service."""
        cmd = self.cmd(f"shell am stopservice {service}")
        return exec_cmd(cmd)

    def input_keyevent(self, keycode):
        """Send keyevent."""
        cmd = self.cmd(f"shell input keyevent {keycode}")
        return exec_cmd(cmd)

    def get_ip_address(self, interface="wlan0"):
        """Extract IP address from ifconfig."""
        cmd = self.cmd(f"shell ifconfig {interface}")
        result = exec_cmd(cmd)
        if result:
            match_obj = re.search(r"inet addr:(.+)  Bcast", result, re.M | re.I)
            if match_obj:
                return match_obj.group(1)
        return "N/A"

    # Other -------------------------------------------------------------------

    def reset(self):
        """Reset."""
        self.serial = None
        self.ip = None

    def destroy(self):
        if self.adb_process:
            self.adb_process.terminate()



# -*- coding: utf-8 -*-

import logging
import subprocess

logger = logging.getLogger(__name__)


class AndroidTVRemote:
    """Simple Android TV Remote control."""

    # https://developer.android.com/reference/android/view/KeyEvent

    def __init__(self, adb):
        self.adb = adb

    # Navigation
    def press_home(self):
        self.adb.input_keyevent("KEYCODE_HOME")

    def press_back(self):
        self.adb.input_keyevent("KEYCODE_BACK")

    def press_dpad_up(self):
        self.adb.input_keyevent("KEYCODE_DPAD_UP")

    def press_dpad_down(self):
        self.adb.input_keyevent("KEYCODE_DPAD_DOWN")

    def press_dpad_left(self):
        self.adb.input_keyevent("KEYCODE_DPAD_LEFT")

    def press_dpad_right(self):
        self.adb.input_keyevent("KEYCODE_DPAD_RIGHT")

    def press_enter(self):
        self.adb.input_keyevent("KEYCODE_ENTER")

    # Volume
    def press_volume_up(self):
        self.adb.input_keyevent("KEYCODE_VOLUME_UP")

    def press_volume_down(self):
        self.adb.input_keyevent("KEYCODE_VOLUME_DOWN")

    def press_volume_mute(self):
        self.adb.input_keyevent("KEYCODE_VOLUME_MUTE")

    # Sleep / Wake
    def press_power(self):
        self.adb.input_keyevent("KEYCODE_POWER")

    def press_soft_sleep(self):
        self.adb.input_keyevent("KEYCODE_SOFT_SLEEP")

    def press_sleep(self):
        self.adb.input_keyevent("KEYCODE_SLEEP")

    def press_wakeup(self):
        self.adb.input_keyevent("KEYCODE_WAKEUP")

    # Input
    def press_tv_input(self):
        self.adb.input_keyevent("KEYCODE_TV_INPUT")

    def press_tv_input_hdmi1(self):
        self.adb.input_keyevent("KEYCODE_TV_INPUT_HDMI_1")

    def press_tv_input_hdmi2(self):
        self.adb.input_keyevent("KEYCODE_TV_INPUT_HDMI_2")

    def press_tv_input_hdmi3(self):
        self.adb.input_keyevent("KEYCODE_TV_INPUT_HDMI_3")

    def press_tv_input_hdmi4(self):
        self.adb.input_keyevent("KEYCODE_TV_INPUT_HDMI_4")



#!/usr/bin/env python

"""The setup script."""

import os
import re
import sys

from setuptools import find_packages, setup


def get_version(*file_paths):
    """Retrieves the version from androidtvremote/__init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(
        r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M
    )  # noqa
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


version = get_version("androidtvremote", "__init__.py")

readme = open("README.md").read()
history = open("HISTORY.md").read()
requirements = open("requirements.txt").readlines()
# if sys.platform == "win32":
#     requirements += open("requirements_windows.txt").readlines()
test_requirements = open("requirements_test.txt").readlines()

setup(
    author="Tim Santor",
    author_email="tsantor@xstudios.com",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    description="Android TV remote for Python.",
    entry_points={
        # "console_scripts": [
        #     "air-methods=androidtvremote.cli:run",
        #     "air-methods-udp=androidtvremote.cli:run_udp",
        # ],
    },
    install_requires=requirements,
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/markdown",
    # data_files=[
    #     (
    #         "androidtvremote",
    #         [
    #             "androidtvremote/data/filename.ext",
    #         ],
    #     )
    # ],
    include_package_data=True,
    keywords="remote android python",
    name="android-tv-remote",
    packages=find_packages(include=["androidtvremote", "androidtvremote.*"]),
    test_suite="tests",
    tests_require=test_requirements,
    url="https://bitbucket.org/xstudios/air-methods-middleman",
    version=version,
    zip_safe=False,
)
